#***********************************************************************

# Licensed Material - Property of IBM

#

# 5724-M24

#

# (c) Copyright IBM Corp. 2008.  All rights reserved.

#

# US Government Users Restricted Rights - Use, duplication or

# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

#************************************************************************



#####################################################################################

# Deploys Websphere Business Services Fabric application on cluster.

# 

# wsadmin <wsadmin options> -f fabricSIBConfig.py  [-cluster clusterName |-node nodeName -server serverName]

#                                                  [-toolsUserid userId]

#

# If no parameters are provided, all required information will be requested 

# on the command line by the script.

#

# The clusterName is name of the Application cluster where fabric enterprise application

# would be deployed.

# 

# The -toolsUserid value is the userid who would have have access to Fabric Adminstration console.

# The userid needs to a valid user in websphere realm.

#

# NOTE: when supplying filestore locations as parameters '/' should be used as 

# the path separator rather than '\', even on Windows platforms.

#

# The script would install the 4 fabric enterprise application on cluster

#  

#############################################################





import sys

import os

import java





##########################################################

# Prints usage error 

###########################################################

def usageError(error):

    print ""

    print "*************************************************************"

    print "Incorrect usage -" + error

    print "*************************************************************"

    print ""

    print "wsadmin <wsadmin options> -f fabricAppDeploy.py  [-cluster clusterName |-node nodeName -server serverName]"

    print "                                                 [-toolsUserid userId]"

    print ""

    print ""

    sys.exit()





###########################################################

# Prints error message on the console

###########################################################

def validationError (errorMessage) :

    print ""

    print "*********************************************"

    print "Error -"

    print errorMessage

    print "*********************************************"

    print ""

    sys.exit()



############################################################

#Get Cluster names in the cell.

############################################################

def getClusterNames(cell) :

    clusterlists = AdminConfig.list("ServerCluster", cell).splitlines()

    clusterList = []

    for clid in clusterlists:

	i = clid.find('(')

        name = clid[0:i]

        clusterList.append(name)

    return clusterList



#-----------------------------------------------------------------------------------------------------

# update fabric rest services

#-------------------------------------------------------------------------------------------------------



def updateRestServices(restServiceTypes, nodeName, serverName, clusterName):

    

    for restServiceType in restServiceTypes :

        attrs = ['-version', '6.2.0.0', '-enable', 'true']

        if clusterName == '':

            attrs.extend(["-nodeName", nodeName, "-serverName", serverName])

        else:

            attrs.extend(["-clusterName", clusterName])

        

        attrs.extend(['-type', restServiceType])

        print "Updating rest service: " + restServiceType

        AdminTask.updateRESTGatewayService(attrs)

        

        

def setDeploymentNode(wizardModel):

    

    cellName = wizardModel["cellName"]

    nodeList = AdminConfig.getid("/Cell:" + cellName + "/Node:/").splitlines()

    

    if(len(nodeList) == 0) :

        validationError("No Node exist in cell " + cellName)

    

    print "Available nodes :"

    nodeNumbers = []

    while(True):

         counter = 0

         for node in nodeList :

             counter = counter + 1

             print counter , "." , str(AdminConfig.showAttribute(node, 'name'))

             nodeNumbers.append(counter)

         

         tagetNode = raw_input("Select the target Node:")

         

         if(tagetNode.isdigit()):

             if(int(tagetNode) in nodeNumbers):

                 wizardModel["nodeId"] = nodeList[int(tagetNode) - 1]

                 wizardModel["nodeName"] = AdminConfig.showAttribute(nodeList[int(tagetNode) - 1], 'name')

                 break

         

         print  "Please select valid input."

         nodeNumbers = []    



def setDeploymentTargetType(wizardModel):

    

    targetTypeMap = {"1":"server", "2":"cluster"}

    

    while(True) :

        print "1. Server"

        print "2. Cluster"

        targetType = raw_input("Select the target environment : ")

             

        if(targetType in targetTypeMap.keys()):

            wizardModel["targetType"] = targetTypeMap[targetType]

            break

        else :

            print "Please select the valid input."

         

def setDeploymentCell(wizardModel):

    

    targetList = AdminConfig.getid("/Cell:/").splitlines()

    

    if(len(targetList) == 0) :

        validationError("No cell exist.")

        

    if(len(targetList) == 1) :

        wizardModel["cellId"] = targetList[0]

        wizardModel["cellName"] = AdminConfig.showAttribute(targetList[0], 'name')

        print "Selecting default cell "+wizardModel["cellName"]

        return

    

    print "Available cells:"

    targetNumbers = []

    while(True):

         counter = 0

         for target in targetList :

             counter = counter + 1

             print counter , "." , str(AdminConfig.showAttribute(target, 'name'))

             targetNumbers.append(counter)

         

         tagetUser = raw_input("Select the target cell:")

         

         if(tagetUser.isdigit()):

             if(int(tagetUser) in targetNumbers):

                 wizardModel["cellId"] = targetList[int(tagetUser) - 1]

                 wizardModel["cellName"] = AdminConfig.showAttribute(targetList[int(tagetUser) - 1], 'name')

                 break

         

         print  "Please select valid input."

         targetNumbers = []    



def setDeploymentServer(wizardModel):

    

    cellName = wizardModel["cellName"]

    nodeName = wizardModel["nodeName"]



    targetList = AdminTask.listServers('[-serverType APPLICATION_SERVER -nodeName '+nodeName+']').splitlines()

    if(len(targetList) == 0) :

        validationError("No Server exist.")

        

    if(len(targetList) == 1) :

        wizardModel["serverId"] = targetList[0]

        wizardModel["serverName"] = AdminConfig.showAttribute(targetList[0], 'name')

        print "Selecting default server "+wizardModel["serverName"]

        return

    

    print "Available servers:"

    

    targetNumbers = []

    while(True):

         counter = 0

         for target in targetList :

             counter = counter + 1

             print counter , "." , str(AdminConfig.showAttribute(target, 'name'))

             targetNumbers.append(counter)

         

         tagetUser = raw_input("Select the target server:")

         

         if(tagetUser.isdigit()):

             if(int(tagetUser) in targetNumbers):

                 wizardModel["serverId"] = targetList[int(tagetUser) - 1]

                 wizardModel["serverName"] = AdminConfig.showAttribute(targetList[int(tagetUser) - 1], 'name')

                 break

         

         print  "Please select valid input."

         targetNumbers = []    



def setDeploymentCluster(wizardModel):

    

    cellName = wizardModel["cellName"]



    targetList = AdminConfig.getid("/Cell:" + cellName + "/ServerCluster:/").splitlines()

    

    if(len(targetList) == 0) :

        validationError("No cluster exist.")

        

    if(len(targetList) == 1) :

        wizardModel["clusterId"] = targetList[0]

        wizardModel["clusterName"] = AdminConfig.showAttribute(targetList[0], 'name')

        print "Selecting default clusters "+wizardModel["clusterName"]

        return

    

    print "Available clusters:"

    

    targetNumbers = []

    while(True):

         counter = 0

         for target in targetList :

             counter = counter + 1

             print counter , "." , str(AdminConfig.showAttribute(target, 'name'))

             targetNumbers.append(counter)

         

         tagetUser = raw_input("Select the target clsuter:")

         

         if(tagetUser.isdigit()):

             if(int(tagetUser) in targetNumbers):

                 wizardModel["clusterId"] = targetList[int(tagetUser) - 1]

                 wizardModel["clusterName"] = AdminConfig.showAttribute(targetList[int(tagetUser) - 1], 'name')

                 break

         

         print  "Please select valid input."

         targetNumbers = []  

    

def setAdminId(wizardModel) :

         

     while(True):

         userId = raw_input("Enter the userid that would have access to Fabric Adminstration console : ")       



         if(userId.strip() == ""):

              print "Please enter valid user name."

         else :

              wizardModel["userId"] = userId

              break    

#------------------------------------------------------------------------------

# check whether the specified application is installed 

#------------------------------------------------------------------------------



def checkApplicationExist(appName, scope):

    print("**************** checkApplicationExist(" + appName + ") start ****************");

    result = False

    applications = AdminApp.list(scope).split()

    for application in applications:

        if (application == appName):

            result = True

            print "appName exists:",appName

            print("**************** checkApplicationExist() end:" + str(result) + " ****************");

            return result

    print("**************** checkApplicationExist() end:" + str(result) + " ****************");

    print "appName doesn't exists:",appName

    return result



#---------------------------------------------------------------------------------------

#Check the for Business Space Cluster

#---------------------------------------------------------------------------------------

def getClusterForBS(cellName):

    

    print "Searching for Business Space install target.."    

    targetList = AdminConfig.getid("/Cell:" + cellName + "/ServerCluster:/").splitlines()

    

    for target in targetList :

        clusterName  = AdminConfig.showAttribute(target, 'name')

        isBSCluster = AdminTask.getBusinessSpaceDeployStatus(["-clusterName", clusterName]) 

        if(isBSCluster == "true"):

            print "Business Space Target is :"+clusterName

            return clusterName

    

    print "No target found."

    return "" 

         



#-----------------------------------------------------------------

# Check if Fabric Widgets  has been configured:

#-----------------------------------------------------------------

def installFabricWidgets(cellName):

    

    installtionRequired = False

    installOptions = []

    

    WAS_INSTALL_DIR = java.lang.System.getProperty("was.install.root")

    widgetPath = WAS_INSTALL_DIR+"/BusinessSpace/widgets/WBSF/"

    

    clusterName = getClusterForBS(cellName)

    scopeAtt = 'WebSphere:cell=' + cellName + ',cluster=' + clusterName        

    

    fabric_widget_app_name  = "widgets_fabric_"+clusterName

            

    scopeAtt = 'WebSphere:cell=' + cellName + ',cluster=' + clusterName

    

    installtionRequired = not checkApplicationExist(fabric_widget_app_name, scopeAtt) and not clusterName == ""             

    

    if(installtionRequired):

        installOptions = ["-clusterName",clusterName,"-widgets",widgetPath]

        print "Installing Fabric widgets."

        AdminTask.installBusinessSpaceWidgets(installOptions)

	print "Fabric widgets installation completed."

    else:

        print "Fabric widget already installed.."

     

############################################################

# Variables

############################################################



#Conditional variables

True = 1 == 1

False = 1 == 0

interactive = False

doCluster = False



userId = ""

password = ""

cellType = ""

cellName = ""

cellId = ""

dmgrPath = ""

fabricBusAuthAlias = ""

queryStrings = []



#EAR Path

catalogEarPath = ""

toolsEARPath = ""

toolsHelpEARPath = ""

traceEARPath = ""





clusterName = ""

clusterId = ""



serverName=""

serverId = ""



nodeName = ""

nodeId = ""



#Line Separator

newLine = "\r"



###################################################################

# Check if the script is being run interactively

###################################################################

if ((len(sys.argv) == 0)):

	interactive = True

        

###################################################################

# Command line parameters passed in. Evaluate the parameters

###################################################################

else:

    nextArg = 0;

    while(len(sys.argv) > nextArg) :

        param = sys.argv[nextArg]

        nextArg += 1



        if (len(sys.argv) <= nextArg):

            usageError("specify value for" + param + " parameter")

        value = sys.argv[nextArg]

        nextArg += 1

        

	if(param == "-cluster"):

	        if(serverName.strip() != "" and nodeName.strip() != ""):

	            usageError("specify the either cluster name or node and server name")

	        clusterName = value  

	    

	elif (param == "-toolsUserid"):

	        userId = value

	    

	elif (param == "-node"):

	        if(clusterName.strip() != ""):

	            usageError("specify the either cluster name or node and server name")

	        nodeName = value

	            

	elif (param == "-server"):

	        if(clusterName.strip() != ""):

	            usageError("specify the either cluster name or node and server name")

	        serverName = value

	    

	else :

	         usageError("Unknown parameter " + param)



###################################################################

# Validate paramter values

###################################################################

    

    #------------------------Only Cluter OR node and server is needed ---------------------->

    

    if(clusterName.strip() == "" and (serverName.strip() == "" or nodeName.strip == "")):

           usageError("specify the either cluster name or node and server name")    

    #--------------------------------------------------------------------------------------->

                

    #----------------------------------------Check if the any cell exist ------------------->    

    cells = AdminConfig.list("Cell").split(lineSeparator)

    if(len(cells) > 1):

        if(len(cellName) < 1):

            usageError("specify the cell name")

    else :

        try :

            cellId = cells[0]

        except :

            print "The profile doesn't have any cell configured."



        i = cellId.find('(')

        cellName = cellId[0:i]



        if(cellId == ""):

	        validationError("cell " + cellName + " doesn't exist")

    #---------------------------------------------------------------------------------------->

    

    #-----------------------------Verify the targets if exists ------------------------------>

    if(not clusterName.strip() == ""):

        clusterId = AdminConfig.getid("/ServerCluster:" + clusterName + "/")

        if(clusterId == ""):

	           validationError("Cluster " + clusterName + "doesn't exist.")

    else :

        serverId = AdminConfig.getid("/Server:" + serverName + "/")

        if(serverId == ""):

            validationError("Server " + serverName + "doesn't exist.")

	    

        nodeId = AdminConfig.getid("/Node:" + nodeName + "/")

        if(nodeId == ""):

            validationError("Node " + nodeName + "doesn't exist.")

    #---------------------------------------------------------------------------------------->

                

    #------------------------Validate the user id ------------------------------------------->    

    if (len(userId) < 1) :

	    usageError("Specify the user id for access to Fabric Admin console.")

    #---------------------------------------------------------------------------------------->

  

  

###################################################################

# End of validation

###################################################################



###################################################################

# If script is being run interactively, query for values.

###################################################################



if (interactive):

    wizardModel = {}    

    setDeploymentTargetType(wizardModel)

    setDeploymentCell(wizardModel)

    

    cellName = wizardModel["cellName"]

    cellId = wizardModel["cellId"]

    

    if(wizardModel["targetType"] == "cluster"):

        setDeploymentCluster(wizardModel)

        clusterName = wizardModel["clusterName"]

        clusterId = wizardModel["clusterId"]

    else:

        setDeploymentNode(wizardModel)

        setDeploymentServer(wizardModel)

        

        nodeName = wizardModel["nodeName"]

        nodeId = wizardModel["nodeId"]

        

        

        serverName = wizardModel["serverName"]

        serverId = wizardModel["serverId"]

            

    setAdminId(wizardModel)

    userId = wizardModel["userId"]    

    

#    print "Input Data:"+str(wizardModel)       

#end-if



WAS_INSTALL_DIR = java.lang.System.getProperty('was.install.root')

installableApps = WAS_INSTALL_DIR + "/installableApps/"



catalogEarPath = installableApps + "catalog.ear"



if (os.path.exists(catalogEarPath) == 0) :

    validationError("Cannot find catalog.ear file. Make sure that ear exists in WAS_INSTALL_DIR/installableApps/ directory.")        



toolsEARPath = str(installableApps) + "tools.ear"

if (os.path.exists(toolsEARPath) == 0) :

    validationError("Cannot find tools.ear file. Make sure that ear exists in WAS_INSTALL_DIR/installableApps/ directory.")        



#toolsHelpEARPath = str(installableApps) + "tools-help.ear"

#if (os.path.exists(toolsHelpEARPath) == 0) :

#    validationError("Cannot find tools-help.ear. Make sure that ear exists in WAS_INSTALL_DIR/installableApps/ directory.") 



traceEARPath = str(installableApps) + "trace.ear"

if (os.path.exists(traceEARPath) == 0) :

    validationError("Cannot find trace.ear. Make sure that ear exists in WAS_HOME/installableApps/ directory.") 



#Set the DA Event Tracer. Todo check for already existing



bindingTarget = ""



if(clusterId != ""):

    bindingTarget = clusterId

else :

    bindingTarget = cellId

    



attr1 = ["name", "wbsf.da.executionTrace.viewer.enable"]

attr2 = ["nameInNameSpace", "wbsf.da.executionTrace.viewer.enable"]

attr3 = ["stringToBind", "true"]



options = []

options.append(attr1)

options.append(attr2)

options.append(attr3)



AdminConfig.create("StringNameSpaceBinding", bindingTarget, options)





attr1 = ["name", "wbsf.da.executionTrace.maxcount"]

attr2 = ["nameInNameSpace", "wbsf.da.executionTrace.maxcount"]

attr3 = ["stringToBind", "100"]



options = []

options.append(attr1)

options.append(attr2)

options.append(attr3)

AdminConfig.create("StringNameSpaceBinding", bindingTarget, options)





if(clusterId != ""):

    #Create the Fabric Domain Replication

    name = ["name", "Fabric DA Replication"]

    options = []

    options.append(name)

    ds = AdminConfig.create("DataReplicationDomain", cellId, options)



    #Target Data Replication for Fabric Domain Replication

    replicas = ["numberOfReplicas", "-1"]

    options = []

    options.append(replicas)

    AdminConfig.create("DataReplication", ds, options)





#Create the Fabric Context Cache

cacheOption1 = ["name", "Fabric Context Cache"]

cacheOption2 = ["jndiName", "services/cache/wbsf.contexts"]

cacheOption3 = ["enableDiskOffload", "True"]

cacheOption4 = ["replicationType", "PUSH_PULL"]

cacheOption5 = ["pushFrequency", "0"]

cacheOption6 = ["diskCacheEntrySizeInMB", "2000"]

cacheOption7 = ["enableCacheReplication", "True"]



if(clusterId != ""):

    cacheProvider = AdminConfig.getid("/ServerCluster:" + clusterName + "/CacheProvider:CacheProvider/")

else :

    cacheProvider = AdminConfig.getid("/Cell:" + cellName + "/CacheProvider:CacheProvider/")



cacheOptions = []

cacheOptions.append(cacheOption1)

cacheOptions.append(cacheOption2)

cacheOptions.append(cacheOption3)

cacheOptions.append(cacheOption4)

cacheOptions.append(cacheOption5)

cacheOptions.append(cacheOption6)

cacheOptions.append(cacheOption7)



objectInstance = AdminConfig.create("ObjectCacheInstance", cacheProvider, cacheOptions)





#Create DR settings for Fabric Context Cache

if(clusterId!=""):

    drOption1 = ["messageBrokerDomainName", "Fabric DA Replication"]

    drOptions = []

    drOptions.append(drOption1)

    AdminConfig.create("DRSSettings", objectInstance, drOptions)





print "Deploying Fabric enterprise applications."



#Ear install

installTarget = []



appExtension = ""

if(clusterId != ""):

    appExtension = "_"+clusterName 



if(clusterId != ""):

    installTarget.extend(["-cluster", clusterName])

else :

    installTarget.extend(["-node", nodeName, "-cell" , cellName, "-server", serverName])





appOptions = ["-appname", "Fabric_Catalog"+appExtension]

appOptions.extend(installTarget)

AdminApp.install(catalogEarPath, appOptions)



#appOptions = ["-appname", "Fabric_Tools_Help"+appExtension]

#appOptions.extend(installTarget)

#AdminApp.install(toolsHelpEARPath, appOptions)



appOptions = ["-appname", "Fabric_Trace"+appExtension]

appOptions.extend(installTarget)

AdminApp.install(traceEARPath, appOptions)



nameOpt = ["-appname", "Fabric_Tools"+appExtension]

mapUserValue = [["FabricAdministrator", "No", "No" , userId , ""]]

mapUserOpt = ["-MapRolesToUsers", mapUserValue] 



appOptions = []

appOptions.extend(nameOpt)

appOptions.extend(mapUserOpt)

appOptions.extend(installTarget)



AdminApp.install(toolsEARPath, appOptions)



print "Deployed Fabric enterprise applications."



restServiceTypes = ['{com.ibm.bpm}BSLM', '{com.ibm.bpm}Glossary', '{com.ibm.bpm}Governance', '{com.ibm.bpm}BVARS']



try :

    updateRestServices(restServiceTypes, nodeName, serverName, clusterName)

    print "Successfully updated Fabric rest services."

except:

    print "Failed to update Fabric rest services."

    

try :

    installFabricWidgets(cellName)

except :

    print " Failed to Install Fabric widgets. Please refer to infocenter documentation to install it manually."



AdminConfig.save()





##########Perform Sync###############################



#Sync node

nodeList = AdminConfig.list('Node').splitlines()

for n in nodeList:

    nodeid = n

    i = nodeid.find('(')

    nodename = nodeid[0:i]

    objn = "type=NodeSync,node=" + nodename + ",*"

    try :

        Sync = AdminControl.completeObjectName(objn)

        if Sync != "":

            AdminControl.invoke(Sync, 'sync')

            print "Done with synchronization for node " + nodename

        continue

    except :

        print "AdminControl not available.Skipping " + nodename + " synchronization."



#Sync code complete

	

try :

    #Starting application install targets

        

    appNames = ["Fabric_Catalog", "Fabric_Tools", "Fabric_Trace"]

        

    if clusterId == "":

        adminCtrl = AdminControl.completeObjectName("cell="+cellName+",node="+nodeName+",type=ApplicationManager,process="+serverName+",*")

    	for appName in appNames :

       		AdminControl.invoke(adminCtrl, 'startApplication', appName+appExtension)

    

    else:

    	 member_ids = AdminConfig.showAttribute(clusterId, "members")

    	 # Remove extra brackets

    	 member_ids = member_ids[1:-1]

    	 # For each member in cluster

    	 for member_id in member_ids.split():

        	member=AdminConfig.showAttribute(member_id, "memberName")

        	node=AdminConfig.showAttribute(member_id, "nodeName")

        	#Formulate the queryString

        	queryString = "cell="+cellName + ",node=" + node +",type=ApplicationManager,process="+ member +",*"

        	queryStrings.append(queryString)

   	     #end-for



    	 # For each query in queryStrings

    	

    	 for queryStr in queryStrings:

        	#Get admin control object

        	adminCtrl = AdminControl.queryNames(queryStr)

        	if adminCtrl != "":

        		for appName in appNames :

       				AdminControl.invoke(adminCtrl,'startApplication',appName+appExtension)	    		

    	 #end-for	    

except :

    print "Fabric enterprise applications installed."

    print "Please start the Fabric enterprise applications manually using WPS Administration console."

      



############################################################################

