##########################################################################
# The scripts configures the datasource
############################################################################

##################################################################
# Creates Datasources and Auth Alias for Fabric database
# for the Websphere Business Services Fabric on Node.
# 
# wsadmin <wsadmin options> -f createDataSources.py    
#                          [-node nodeName]
#						   [-server serverName] 
#                          [-fabricDbUser dbUser]
#						   [-fabricDbPassword fabricDbPassword] 
#						   [-fabricDbName fabricDBName] 
#                          [-fabricDbServerName  fabricDbServerName]
#                          [-fabricDbportNumber fabricDbportNumber]
#                          [-fabricDbJDBCClassPath fabricDbJDBCClassPath]
#                          [-fabricDbDriverType fabricDbDriverType]
#                          [-fabricDbType fabricDbType]
#                          [-fabricDbSchemaName fabricDbSchemaName]
#                          [-fabricJBBCProviderScope fabricJBBCProviderScope]
# 
# The fabricDbSchemaName parameter is required only for zos.
###################################################################

import sys
import java


##########################################################
# Prints usage error 
###########################################################

def usageError(error):
    sys.stderr.write(error)
    print "Exception:Incorrect usage -" + str(error)
    sys.exit()

###########################################################
# Prints error message on the console
###########################################################
def validationError (errorMessage) :
    sys.stderr.write(errorMessage)
    print "Exception:validationError - " + str(errorMessage)
    sys.exit()
 

##########################################################
#Get Datasources based on JNDIname and scope.
##########################################################
def findDataSource(jndiName,scopeId,scope):

    dataSourceList = AdminConfig.list("DataSource", scopeId).split(lineSeparator)
    for dsentry in dataSourceList :
        dsJndiName = AdminConfig.showAttribute(dsentry,"jndiName")
        if(dsJndiName == jndiName) :
	        provider = AdminConfig.showAttribute(dsentry,"provider")
	        provideName = AdminConfig.showAttribute(provider,"name")
	        dataSourceName = AdminConfig.showAttribute(dsentry,"name")
	        path = scope + "JDBCProvider:"+ provideName + "/DataSource:" + dataSourceName+"/"
	        dataSource = AdminConfig.getid(path)
	        if(dataSource != "") :
	            return dataSource
    return ""


##########################################################
#Get JASSAuthData based on Auth Alias
##########################################################

def getJAASAuthData(aliasAuth) :
    authObj = ""
    aliasidlist = AdminConfig.list("JAASAuthData").split(lineSeparator)
    for aliasid in aliasidlist:
        qualifiedaliasName = AdminConfig.showAttribute(aliasid,"alias")
        i= qualifiedaliasName.find('/')
        name=qualifiedaliasName[i+1:len(qualifiedaliasName)]
        if (name==aliasAuth):
            authObj = aliasid
            break
    return authObj

##############################################################
# Get JDBC Provider Configuration
#############################################################

def getJdbcProviderConfigId(scope, providerName):
    providerConfigId = AdminConfig.getid(scope + "JDBCProvider:" + providerName + "/").splitlines()
    if (providerConfigId == ""):
        print "Did not find JDBC Provider: " + providerName
        return None
    else:
        for provider in providerConfigId:
            print "provider" +str(provider)
            if((provider.find(providerName) >= 0) and (provider.find("builtin_jdbcprovider") < 0)):
                print "if provider" +str(provider) 
                return provider
        

def getJdbcTemplateId(templateName):
    templateList = AdminConfig.listTemplates('JDBCProvider',templateName).splitlines()
    for temp in templateList :
        if ((AdminConfig.showAttribute(temp,"name") ==  templateName) and (AdminConfig.showAttribute(temp,"providerType") == templateName)) :
            print "Found JDBCProvider Template" + temp
            return temp
        

##########################################################
#Create JAASAuthData
##########################################################

def createJAASAuthData(alias,userId,password):
    authObj = getJAASAuthData(alias)
    aliasAttr = ['alias', alias]
    useridAttr = ['userId', userId]
    passwordAttr = ['password', password]
    jaasAttrs = [aliasAttr, useridAttr, passwordAttr]
    if(len(authObj) > 1):
        #Update the values
        print "Found Alias" + str(alias)
        AdminConfig.modify(authObj,jaasAttrs)
    else :
        security = AdminConfig.getid('/Cell:'+ cellName +'/Security:/')
        #Create Auth Alias
        print "Created J2C Auth" + alias
        AdminConfig.create('JAASAuthData', security, jaasAttrs)

##########################################################
#Get websphere variables based on scope id : Cell id /Node id /Server id /Cluster id
##########################################################
def getWebsphereVariable(variablename,scope) :
    variableEntry = AdminConfig.list("VariableSubstitutionEntry",scope).split(lineSeparator)
    value = "";
    for ventry in variableEntry :
        name = AdminConfig.showAttribute(ventry,"symbolicName")
        if(name == variablename):
            value = AdminConfig.showAttribute(ventry,"value")
            break
    return value

def createDataSource(providerId,template,name,jndiName,helperName,authAlias):
    nameAttr = ["name", name]
    jndiAttr = ["jndiName", jndiName]
    statementCacheSizeAttr = ["statementCacheSize","100"]
    helpAttr = ["datasourceHelperClassname", helperName]
    authAttr = ["authDataAlias", authAlias]
    datasourceCreateArgs = [nameAttr, jndiAttr, statementCacheSizeAttr, helpAttr, authAttr]
    return AdminConfig.createUsingTemplate('DataSource', providerId, datasourceCreateArgs,template)

def createUpdateFabricDataSource(providerId,templateDSId,name,jndi,datasourceHelperClassname,fabricJ2CAuth,scopeId,scope,customProps) :
    ds = findDataSource(jndi,scopeId,scope)
    if(ds == "") :
        print "Creating Datasource with JNDI Name at scope - " + scopeId
        ds = createDataSource(providerId,templateDSId,name,jndi,datasourceHelperClassname,fabricJ2CAuth)
    else :
        print "Existing Datasource found with JNDI Name " + str(jndi) + " at scope - " + scopeId
        #User would have changed dbTypes, hence removing old one.
        AdminConfig.remove(ds)
        print "Creating new Datasource with JNDI Name " + str(jndi) + " at scope - " + scopeId
        ds = createDataSource(providerId,templateDSId,name,jndi,datasourceHelperClassname,fabricJ2CAuth)
      
    ps =  AdminConfig.showAttribute(ds,"propertySet")
    rps = AdminConfig.showAttribute(ps,"resourceProperties")
    rpsarray = rps[1:len(rps)-1].split(' ')
    
    if(len(customProps)>0):
        for rp in rpsarray:
            name = AdminConfig.showAttribute(rp, 'name')
            if customProps.has_key(name) :
                AdminConfig.modify(rp, [['value', customProps[name]]])
    
            
    

##############################################################
# Converts to List
##############################################################
def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split() #splits space-separated lists,
        else:
                tmpList = inStr.split("\n")   #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();         #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

############################################################
# Environment variables
############################################################    

def createWebsphereEnvironment(cellNm,nodeNm,varName,varValue):
    print "Entering createWebsphereEnvironment"
    print "varName="+varName+",varValue="+varValue
    
    id = ""
    variableMapNodeId = ""

    if(nodeNm == ""):    
        id = AdminConfig.getid("/Cell:"+cellNm+"/")
        variableMapNodeId = AdminConfig.getid("/Cell:"+cellNm+"/VariableMap:/")
    else:
        id = AdminConfig.getid("/Cell:"+cellNm+"/Node:"+nodeNm+"/")
        variableMapNodeId = AdminConfig.getid("/Cell:"+cellNm+"/Node:"+nodeNm+"/VariableMap:/")
               

    wasVarName = ["symbolicName", varName]
    wasVarValue = ["value", varValue]
    wasVarAttrs = [wasVarName, wasVarValue]

    installed    = "false"
    modified     = "false"

    varList = AdminConfig.list("VariableSubstitutionEntry", id )
    for tmpEntryId in wsadminToList(varList):
        #For Cell scope, we skip nodes
        if(nodeNm == ""):
            if(str(tmpEntryId).find("/nodes/") != -1):
                continue
            
        name = AdminConfig.showAttribute(tmpEntryId, "symbolicName")
        value = AdminConfig.showAttribute(tmpEntryId, "value")
        if (cmp(name, varName) == 0):
            installed = "true"
            if (cmp(value, varValue) != 0):
                print  varName + "needs to be changed"
                AdminConfig.modify(tmpEntryId, wasVarAttrs )
                value = AdminConfig.showAttribute(tmpEntryId, "value" )
                modified = "true"
            #endIf 
            break
        #endIf 
    #endFor 


    flagOk = "false"
    if (cmp(modified, "false") == 0):
        if (cmp(installed, "true") == 0):
            flagOk = "true"
            print varName + " with the correct value is already there, no action taken."
        else:
            print varName + " needs to be created."
            variableEntryId = AdminConfig.create("VariableSubstitutionEntry", variableMapNodeId, wasVarAttrs )
            name = AdminConfig.showAttribute(variableEntryId, "symbolicName" )
            value = AdminConfig.showAttribute(variableEntryId, "value" )
        #endElse 
    #endIf 

    # Check for success
    if (cmp(modified, "true") == 0 or cmp(installed, "false") == 0):
        varList = AdminConfig.list("VariableSubstitutionEntry", id )
        for tmpEntryId in wsadminToList(varList):
            #For Cell scope, we skip nodes
            if(nodeNm == ""):
                if(str(tmpEntryId).find("/nodes/") != -1):
                    continue
            name = AdminConfig.showAttribute(tmpEntryId, "symbolicName" )
            value = AdminConfig.showAttribute(tmpEntryId, "value" )
            if (cmp(name, varName) == 0):
                if (cmp(value, varValue) == 0):
                    flagOk = "true"
                #endIf 
                break
            #endIf 
        #endFor 
    #endIf         
    if (cmp(flagOk, "true") == 0):
        print varName + " successfully created/modified."
    else :
        print "Warning : WebSphere Environment Variable "+varName+" could not be created/modified. You need to manually set the variable"    
        
    print "Exiting createWebsphereEnvironment"
    return
#endDef

def isZnode(cellName , nodeName):
    
    # Consider the first node of the cell if node name is not provided
    
    if(nodeName.strip()==""):
        if(cellName.strip()==""):
            print "Error:Can't determine the OS of the node"
            sys.exit();
        else:
            cellId = AdminConfig.getid("/Cell:"+cellName+"/")
        
        nodes = AdminConfig.list("Node",cellId).split(lineSeparator)
        nodeName = AdminConfig.showAttribute(nodes[0],"name")
        print "Using First Node"+nodeName
    
    # Return the true of the node is on z/os 
    
    if(AdminTask.isNodeZOS(['-nodeName',nodeName]).upper() == "TRUE" ):
        return "TRUE"
    else:
        return "FALSE"
    
############################################################
# Variables
############################################################

#Constants for Boolean types
True = 1==1
False = 1==0
isVMMConfigured = True
isProviderNew = False
startTime = java.lang.System.currentTimeMillis()
nodeName = ""
fabricDbUser = ""
fabricDbPassword = ""
fabricDbName = ""
fabricDbServerName = ""
fabricDbportNumber = ""
fabricDbJDBCClassPath = ""
fabricDbDriverType = ""
fabricDbType = ""
serverName = ""
cellId = ""
cellName = ""
serverId = ""
nodeId = ""
template = ""
templateDSName = ""
fabricJBBCProviderScope = ""
fabricDbSchemaName = ""
templateName = ""
scopeId = ""
scope = ""

#JNDI Names
fabricBSRJNDI = "fabric/bsr"
fabricPMJNDI = "fabric/pm"
fabricGMJNDI = "fabric/gm"
fabricJ2CAuth = "FABRIC_JDBC_AUTH"
fabricBSR = "fabric bsr"
fabricBSRJNDI = "jdbc/fabric/bsr"
fabricGM = "fabric gm" 
fabricGMJNDI = "jdbc/fabric/gm"
fabricPM = "fabric pm"
fabricPMJNDI = "jdbc/fabric/pm"
lineSeparator = java.lang.System.getProperty('line.separator')



###################################################################
# Check if the script is being run interactively
###################################################################

if ((len(sys.argv) == 0)):
    validationError("No inputs provided")
        
###################################################################
# Command line parameters passed in. Evaluate each of these paramters
###################################################################
else:
    
    nextArg = 0;
    while(len(sys.argv) > nextArg) :
        param = sys.argv[nextArg]
        nextArg +=1

        if (len(sys.argv) <= nextArg):
            usageError("specify value for" + param  + " parameter")

        value = sys.argv[nextArg]
        nextArg +=1

        if (param == "-node"):
            nodeName = value
        elif (param == "-fabricDbUser"):
            fabricDbUser = value
        elif (param == "-fabricDbPassword"):
            fabricDbPassword = value
        elif (param == "-fabricDbName"):
            fabricDbName = value
        elif (param == "-fabricDbServerName"):
            fabricDbServerName=value
        elif (param == "-fabricDbportNumber"):
            fabricDbportNumber=value
        elif (param == "-fabricDbJDBCClassPath"): 
            fabricDbJDBCClassPath = value
	    fabricDbJDBCClassPath = fabricDbJDBCClassPath.replace("*"," ")
        elif (param == "-fabricDbDriverType"): 
            fabricDbDriverType = value
        elif (param == "-fabricDbType"): 
            fabricDbType = value
        elif (param == "-fabricJBBCProviderScope"): 
            fabricJBBCProviderScope = value
        elif (param == "-fabricDbSchemaName"):
            fabricDbSchemaName = value
        else :
            usageError("Unknown parameter " + param)

###################################################################
# Validate paramter values
###################################################################
    if(len(nodeName) < 1 ):
        usageError("NodeName not provided")
    
    nodeId=AdminConfig.getid("/Node:"+nodeName+"/")
    if (nodeId==""):
        validationError("Node "+ nodeName + "doesn't exist")
    
    if(len(serverName) < 1):
        servers = AdminConfig.list("Server", nodeId)
        serversList = servers.split(lineSeparator)
	serverName =  AdminConfig.showAttribute(serversList[0], "name")
    
    serverId=AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName+"/")
    if (serverId==""):
        validationError("Server "+ serverName + " doesn't exist")

    if(len(fabricDbType) < 1):
        validationError("fabricdbType value not provided")

    if(len(fabricJBBCProviderScope) < 1):
        validationError("scope value not provided")
    
    
    celllist = AdminConfig.list("Cell").split(lineSeparator)
    cellId = celllist[0]
    i= cellId.find('(')
    cellName=cellId[0:i]

    if(cellId == ""):
        validationError("cell " + cellName + " doesn't exist")

    if(fabricJBBCProviderScope.upper() == "CELL"):
        scope = "/Cell:" + cellName +"/"
        scopeId = cellId

    else :
        scope = "/Node:"+nodeName+"/Server:"+serverName+"/"
        scopeId = serverId

    try :
        resourceAttrs = []
        customAttirbMap = {}

        # Create or replace JASS Auth Data
        if(fabricDbType.upper() != "DERBY_EMBEDDED"):
            createJAASAuthData(fabricJ2CAuth,fabricDbUser,fabricDbPassword)
        else :
            fabricJ2CAuth = None

        #Check if datasource exists

        # Create the datasource
        if (fabricDbType.upper() == "DB2_UNIVERSAL"):
            newsprops = []
            
            template = "DB2 Universal JDBC Driver Provider (XA)"
            templateDSName = "DB2 Universal JDBC Driver XA"                
            zNode = isZnode(cellName,nodeName)
            if(zNode == "TRUE"):
                if(len(fabricDbSchemaName) < 1):
                     validationError("Database schema name not provided")   
                
                customAttirbMap["currentSQLID"]=fabricDbSchemaName
                
                if(fabricDbDriverType == "2"):
                    template = "DB2 Universal JDBC Driver Provider"
                    templateDSName = "DB2 Universal JDBC Driver DataSource"             
                                       
            templateDSId = AdminConfig.listTemplates("DataSource", templateDSName)
            datasourceHelperClassname = "com.ibm.websphere.rsadapter.DB2UniversalDataStoreHelper"
            
            ###################################################################################
            #                 Set the J2EE Resource attributes                                #
            ###################################################################################
            
            
            customAttirbMap["databaseName"]=fabricDbName
            customAttirbMap["serverName"]=fabricDbServerName
            customAttirbMap["driverType"]=fabricDbDriverType
            customAttirbMap["portNumber"]=fabricDbportNumber
            
            if(fabricJBBCProviderScope.upper() == "CELL") :
                createWebsphereEnvironment(cellName,"","DB2UNIVERSAL_JDBC_DRIVER_PATH",fabricDbJDBCClassPath)
            else :
               createWebsphereEnvironment(cellName,nodeName,"DB2UNIVERSAL_JDBC_DRIVER_PATH",fabricDbJDBCClassPath)


        elif(fabricDbType.upper() == "DERBY_EMBEDDED"):
            newsprops = []
            template ="Derby JDBC Provider (XA)"
            templateDSName = "Derby JDBC Driver XA DataSource"
            templateDSId = AdminConfig.listTemplates("DataSource", templateDSName)
            datasourceHelperClassname = "com.ibm.websphere.rsadapter.DerbyDataStoreHelper"
            
            ###################################################################################
            #                 Set the J2EE Resource attributes                                #
            ###################################################################################
            
            customAttirbMap["databaseName"]="${USER_INSTALL_ROOT}/databases/"+fabricDbName
            
        
        elif(fabricDbType.upper() == "ORACLE10G"):
            newsprops = []
            template = "Oracle JDBC Driver (XA)"
            templateDSName = "Oracle JDBC Driver XA"
            templateDSId = AdminConfig.listTemplates("DataSource", templateDSName)
            datasourceHelperClassname = "com.ibm.websphere.rsadapter.Oracle10gDataStoreHelper"
            if(fabricDbDriverType.upper() == "4"): 
                connectString = "jdbc:oracle:thin:@"+fabricDbServerName+":"+fabricDbportNumber+":"+fabricDbName
            else :
                connectString = "jdbc:oracle:oci:@"+fabricDbServerName+":"+fabricDbportNumber+":"+fabricDbName
            
            ###################################################################################
            #                 Set the J2EE Resource attributes                                #
            ###################################################################################
            
            customAttirbMap["URL"] = connectString
            
            if(fabricJBBCProviderScope.upper() == "CELL") :
                createWebsphereEnvironment(cellName,"","ORACLE_JDBC_DRIVER_PATH",fabricDbJDBCClassPath)
            else :
               createWebsphereEnvironment(cellName,nodeName,"ORACLE_JDBC_DRIVER_PATH",fabricDbJDBCClassPath)

           
        #Get Provider Template
        templateId = getJdbcTemplateId(template)

        providerId = getJdbcProviderConfigId(scope,template)

        print "Got providerId" + str(providerId)

        #Create JDBC Provider
        if((providerId == None)):
            #Create Fabric JDBC Provider.
            print "Create Fabric JDBC Provider"
            providerCreateArgs = [["name", template]]
            providerId = AdminConfig.createUsingTemplate("JDBCProvider", scopeId, providerCreateArgs,templateId)
            isProviderNew = True
            

        #Create PM JNDI
        createUpdateFabricDataSource(providerId,templateDSId,fabricPM,fabricPMJNDI,datasourceHelperClassname,fabricJ2CAuth,scopeId,scope,customAttirbMap)

        #Deleting extraneous data source
        if(isProviderNew == True):
            dataSourceId = ""
            defaultJdbcTemplateName = template
            
            # Set the extraneous data source name   	
                   		
            if(fabricDbType.upper() == "ORACLE10G") :                
                defaultDataSourceName = "Oracle JDBC Driver XA DataSource"
                dataSourceId = AdminConfig.getid(scope+"JDBCProvider:"+defaultJdbcTemplateName+"/DataSource:"+defaultDataSourceName+"/")        
            elif(fabricDbType.upper() == "DB2_UNIVERSAL"):
                defaultJdbcTemplateName = template                
                if(zNode == "TRUE"):
                	defaultDataSourceName = "DB2 Universal JDBC Driver DataSource"
                else:                	
	                defaultDataSourceName = "DB2 Universal JDBC Driver XA DataSource"
                dataSourceId = AdminConfig.getid(scope+"JDBCProvider:"+defaultJdbcTemplateName+"/DataSource:"+defaultDataSourceName+"/")    
	              
            if (dataSourceId != "") :
                print "Deleting extraneous data source" + str(dataSourceId)
                AdminConfig.remove(dataSourceId)
         
        print "Fabric datasources configured."

        #Save configuration
        AdminConfig.save()
        endTime = java.lang.System.currentTimeMillis()
        totalTime = (endTime - startTime)/1000
        print "Completed in ",totalTime," seconds."
             
    except Exception,e:
        print "Exception:" + str(e)
        sys.stderr.write("Exception is "+str(sys.exc_info()[1])) 
        sys.exit()
    except :
        print "Exception:" + str(sys.exc_info()[0])
        sys.stderr.write("Exception is "+str(sys.exc_info()[1])) 
        sys.exit()    

     


